(function() {
    // # Globbing
    // for performance reasons we're only matching one level down:
    // 'test/spec/{,*/}*.js'
    // use this if you want to recursively match all subfolders:
    // 'test/spec/**/*.js'

    'use strict';

    module.exports = function (grunt) {

        // Time how long tasks take. Can help when optimizing build times
        require('time-grunt')(grunt);

        // Configurable paths for the application
        var appConfig = {
            app: 'app',
            dist: 'dist',
            tmp: '.tmp',
            root: '.',
            lib: '../library'
        };

        grunt.loadNpmTasks('grunt-contrib-clean');
        grunt.loadNpmTasks('grunt-contrib-copy');
        grunt.loadNpmTasks('grunt-contrib-concat');
        grunt.loadNpmTasks('grunt-contrib-compass');
        grunt.loadNpmTasks('grunt-contrib-cssmin');
        grunt.loadNpmTasks('grunt-contrib-uglify');
        grunt.loadNpmTasks('grunt-contrib-jshint');
        grunt.loadNpmTasks('grunt-usemin');
        grunt.loadNpmTasks('grunt-angular-templates');
        grunt.loadNpmTasks('grunt-banner');

        // Define the configuration for all the tasks
        grunt.initConfig({

            config: appConfig,

            // Validate JS files
            jshint: {
                options: {
                    jshintrc: '.jshintrc',
                    reporter: require('jshint-stylish')
                },
                all: {
                    src: [
                        'Gruntfile.js',
                        '<%= config.app %>/scripts/{,*/}*.js',
                        '<%= config.lib %>/*.js'
                    ]
                }
            },

            // Empties folders to start fresh
            clean: {
                dist: ['<%= config.tmp %>', '<%= config.dist %>/{,*/}*']
            },

            // Copy static files to the dist folder
            copy: {
                dist: {
                    files: [{
                        expand: true,
                        cwd: '<%= config.app %>',
                        src: [
                            '*.{ico,png,txt}',
                            '*.html',
                            '*.md',
                            'images/{,*/}*.*',
                            'fonts/{,*/}*.*'
                        ],
                        dest: '<%= config.dist %>'
                    }, {
                        expand: true,
                        cwd: 'node_modules/font-awesome',
                        src: 'fonts/*',
                        dest: '<%= config.dist %>'
                    }]
                }
            },

            // Compile Sass into CSS
            compass: {
                options: {
                    sassDir: '<%= config.app %>/styles',
                    cssDir: '<%= config.tmp %>/styles',
                    javascriptsDir: '<%= config.app %>/scripts',
                    fontsDir: '<%= config.app %>/styles/fonts',
                    importPath: '<%= config.root %>'
                },
                dist: {}
            },

            // usemin preparation task
            useminPrepare: {
                options: {
                    dest: '<%= config.dist %>',
                    flow: {
                        steps: {
                            libjs: ['concat'],
                            appjs: ['concat'],
                            js: ['concat', 'uglify'],
                            css: ['cssmin']
                        }
                    }
                },
                html: '<%= config.app %>/index.html'
            },

            // Concatenate (and/or) minify Javascript and CSS files from index.html
            usemin: {
                options: {
                    blockReplacements: {
                        libjs: function(block) {
                            return '<script src="' + block.dest + '"></script>';
                        },
                        appjs: function(block) {
                            return '<script src="' + block.dest + '"></script>';
                        }
                    },
                    assetsDirs: [
                        '<%= config.dist %>'
                    ]
                },
                html: '<%= config.dist %>/index.html',
                css: ['<%= config.dist %>/styles/{,*/}*.css'],
                js: ['<%= config.dist %>/scripts/{,*/}*.js']
            },

            // Wrap Angular template files into $templateCache
            ngtemplates: {
                dist: {
                    options: {
                        module: 'weChatApp'
                    },
                    cwd: '<%= config.app %>',
                    src: 'views/{,*/}*.html',
                    dest: '<%= config.tmp %>/templateCache.js'
                }
            },

            // Inject $templateCache into app.js
            concat: {
                ngtemplates: {
                    src: ['<%= config.dist %>/scripts/app.js', '<%= ngtemplates.dist.dest %>'],
                    dest: '<%= config.dist %>/scripts/app.js'
                }
            },

            // Inject versioning header on top of the files
            usebanner: {
                dist: {
                    options: {
                        banner: '/*\n' +
                        'Veeva MyInsights Library version ${version}\n\n' +
                        'http://developer.veevacrm.com/\n\n' +
                        'Copyright \u00A9 2023 Veeva Systems, Inc. All rights reserved.\n\n' +
                        'My Insights Library is dependent on the Q library, which enables you to work with promises as returns from the My Insights Library methods.\n' +
                        'Include the Q library as a script in the custom report package.\n' +
                        'Q Library License Acknolegements\n' +
                        'Copyright 2009-2017 Kristopher Michael Kowal. All rights reserved.\n' +
                        'Q library can be downloaded here https://github.com/kriskowal/q/blob/v1/LICENSE\n' +
                        '*/',
                        linebreak: true
                    },
                    files: {
                        src: ['<%= config.dist %>/scripts/lib.js']
                    }
                }
            }

        });

        grunt.registerTask('test', [
            'jshint'
        ]);

        grunt.registerTask('build', [
            'clean:dist',
            'copy',
            'compass',
            'useminPrepare',
            'concat:generated',
            'cssmin:generated',
            'uglify:generated',
            'ngtemplates',
            'concat:ngtemplates',
            'usemin',
            'usebanner'
        ]);

        grunt.registerTask('default', [
            'test',
            'build'
        ]);
    };
})();